/* Boiler plate CGI program to process FORM input.
 *
 * Written by Paul D. Boyle and uses his cgi_util routines.
 * August 1997.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * Terms of License:
 *
 *                          NO WARRANTY
 *
 * 1. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
 * PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION.
 * 
 * 2. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
 * WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY
 * AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU
 * FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING
 * BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR
 * LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM
 * TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

/* local includes */
#include "cgi_util.h"


#define FIELD_TOKEN_STR  "&"  /* for parsing with strtok() */

/* here starts the main() routine */
int main( void )
{
    size_t  content_length;

    char    *cl_ptr,
            *buffer,
            *ptr,
            *dat_tim,
            *from_who = NULL,
            *from_who_email = NULL;

    int     n_fields,
            i;
            time_t now;
    
    FILE    *registration;

    union  cgi_data *entry;

/* first get the content length from the environment variable */

    cl_ptr = getenv( "CONTENT_LENGTH" );
    if( !cl_ptr ) {
        ReportErrorHTML( "CONTENT_LENGTH variable not set." );
        exit( EXIT_SUCCESS );
    }
    content_length = (size_t)atoi( cl_ptr );

/* check for errors with METHOD and CONTENT_TYPE */

    switch( VerifyForm("POST") ) {
      case 0:   /* everything ok */
         break;
      case 1:
         ReportErrorHTML( "REQUEST_METHOD incorrectly set." );
         exit( EXIT_SUCCESS );
      case 2:
         ReportErrorHTML( "CONTENT_TYPE incorrectly set." );
         exit( EXIT_SUCCESS );
      case 3:
         ReportErrorHTML(
         "REQUEST_METHOD and CONTENT_TYPE incorrectly set." );

         exit( EXIT_SUCCESS );
      default:  /* should never reach here */
         ReportErrorHTML( "Unrecognized Error Condition. Exiting" );
         exit( EXIT_SUCCESS );
    }

/* allocate memory for read in buffer and fill it with data */
    buffer = malloc( content_length + 1 );
    if( !buffer ) {
        ReportErrorHTML( "Read buffer memory allocation error." );
        exit( EXIT_FAILURE );
    }
    buffer[content_length] = '\0';
    fread( buffer, 1, content_length, stdin );


/* get number of fields and dynamically allocate an array of unions for
 * the data.
 */
    n_fields = CountFields( buffer );

    entry = malloc( n_fields * sizeof(union cgi_data) );
    if( !entry ) {
        ReportErrorHTML( "Memory allocation error" );
        exit( EXIT_FAILURE );
    }

/* chop the buffer into data fields and process the name value pairs */
    ptr = buffer;
    i = 0;
    for( ptr = strtok(buffer, FIELD_TOKEN_STR);
        NULL != ptr;
        ptr = strtok(NULL, FIELD_TOKEN_STR )) {
          entry[i].field = malloc( strlen(ptr) + 1 );
         if( !entry[i].field ) {
             ReportErrorHTML( "Memory Allocation Error. Exiting." );
             exit( EXIT_FAILURE );
         }
         strcpy( entry[i].field, ptr);
         PlusToSpace( entry[i].field );
         entry[i].field = TranslateEscapes( entry[i].field );
         entry[i].field = TranslateNewlines( entry[i].field );
         entry[i].pair.name = entry[i].field;
         entry[i].pair.value = ParseNameValuePairs( entry[i].field );
         i++;
    }


/* Summarize for the user */
    fprintf(stdout, "Content-type: text/html\n\n" );
    fprintf(stdout, "<html>\n");
    fprintf(stdout, "<head><title>\n");
    fprintf(stdout, "</title></head><body>");
    fprintf(stdout,"<H1>Summary</H1>");
    fprintf(stdout,
            "The following data was submitted:<p>\n" );

    fprintf( stdout, "<ul>\n" );
    for( i = 0; i < n_fields; i++ )
         fprintf( stdout, "<li>%s = %s\n",
                  entry[i].pair.name, entry[i].pair.value );
    fprintf( stdout, "</ul>\n" );

    fprintf(stdout, "</body></head></html>\n");

    exit( EXIT_SUCCESS );
}
